////
////  AppSDKManager.m
////  blaccount_flutter_plugin
////
////  Created by 王振 on 2021/8/24.
////
//
#import "AppSDKManager.h"
#import <BLLetCore/BLLetCore.h>
#import <BLLetBase/BLLetBase.h>
#import <BLLetFamily/BLLetFamily.h>
#import <BLLetAccount/BLLetAccount.h>
#import <JSONKit-BroadLink/JSONKit.h>


//字符串是否为空
#define BL_IsStrEmpty(_ref)    (((_ref) == nil) || ([(_ref) isEqual:[NSNull null]]) || ([@"" isEqualToString:(_ref)]))

@interface AppSDKManager ()

@property (strong,nonatomic)NSMutableArray <BLDNADevice *>*localDeviceArray;

@end

@implementation AppSDKManager

+(instancetype)sharedManager {
    static dispatch_once_t onceToken;
    static AppSDKManager *manager = nil;
    dispatch_once(&onceToken, ^{
        manager = [[self alloc] init];
    });
    return manager;
}
-(NSMutableArray<BLDNADevice *> *)localDeviceArray {
    if (_localDeviceArray == nil) {
        _localDeviceArray = [NSMutableArray array];
    }
    return _localDeviceArray;
}
-(BOOL)blAppSDKAccountInit {
    if ([BLLet sharedLet].configParam.companyId.length > 0) {
        [BLAccount sharedAccount];
        return YES;
    }else {
        return NO;
    }
}
-(void)loginWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] login:BL_IsStrEmpty([info objectForKey:@"account"]) ?@"" : [info objectForKey:@"account"] password:BL_IsStrEmpty([info objectForKey:@"password"]) ? @"" :[info objectForKey:@"password"] completionHandler:^(BLLoginResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg,
                @"data":@{//nickname,icon_path,sex,country_code,phone,email,birthday,user_type
                        @"login_session":result.loginsession?result.loginsession:@"",
                        @"user_id":result.userid?result.userid:@"",
                        @"nickname":result.nickname?result.nickname:@"",
                        @"icon_path":result.iconpath?result.iconpath:@"",
                        @"sex":result.sex?result.sex:@"",
                        @"country_code":result.countrycode?result.countrycode:@"",
                        @"phone":result.phone?result.phone:@"",
                        @"email":result.email?result.email:@"",
                        @"birthday":result.birthday?result.birthday:@"",
                        @"user_type":result.usertype?result.usertype:@"",
                }
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}

-(void)localLoginWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] localLoginWithUsrid:!BL_IsStrEmpty([info objectForKey:@"user_id"]) ?[info objectForKey:@"user_id"]:@"" session:!BL_IsStrEmpty([info objectForKey:@"login_session"]) ? [info objectForKey:@"login_session"]:@"" completionHandler:^(BLLoginResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
- (NSString *)changePhoneCountryCode:(NSString *)code {
    //手机号国家码统一修改为 00 开头
    if (BL_IsStrEmpty(code)) {
        return @"0086";
    }else if ([code hasPrefix:@"00"]) {
        return code;
    } else if ([code hasPrefix:@"+"]) {
        NSString *sub = [code substringFromIndex:1];
        return [NSString stringWithFormat:@"00%@", sub];
    } else {
        return [NSString stringWithFormat:@"00%@", code];
    }
}
-(void)fastLoginWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    NSString *countrycode;
    if ([info.allKeys containsObject:@"country_code"] && !BL_IsStrEmpty([info objectForKey:@"country_code"])) {
        countrycode = [self changePhoneCountryCode:[info objectForKey:@"account"]];
    }else {
        countrycode = @"";
    }
    [[BLAccount sharedAccount] fastLoginWithPhoneOrEmail:!BL_IsStrEmpty([info objectForKey:@"account"]) ? [info objectForKey:@"account"]:@"" countrycode:countrycode vcode:!BL_IsStrEmpty([info objectForKey:@"vcode"]) ? [info objectForKey:@"vcode"]:@"" logintry:@"false" completionHandler:^(BLLoginResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg,
                @"data":@{//nickname,icon_path,sex,country_code,phone,email,birthday,user_type
                        @"login_session":result.loginsession?result.loginsession:@"",
                        @"user_id":result.userid?result.userid:@"",
                        @"nickname":result.nickname?result.nickname:@"",
                        @"icon_path":result.iconpath?result.iconpath:@"",
                        @"sex":result.sex?result.sex:@"",
                        @"country_code":result.countrycode?result.countrycode:@"",
                        @"phone":result.phone?result.phone:@"",
                        @"email":result.email?result.email:@"",
                        @"birthday":result.birthday?result.birthday:@"",
                        @"user_type":result.usertype?result.usertype:@"",
                }
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
-(void)sendFastVCodeForLoginWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    [[BLAccount sharedAccount] sendFastVCode:!BL_IsStrEmpty([info objectForKey:@"account"]) ? [info objectForKey:@"account"]:@"" countryCode:!BL_IsStrEmpty([info objectForKey:@"country_code"]) ? [info objectForKey:@"country_code"]:nil completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}

-(void)sendPhoneCodeForRegisterWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] sendRegVCode:!BL_IsStrEmpty([info objectForKey:@"phone"]) ? [info objectForKey:@"phone"]:@"" countryCode:!BL_IsStrEmpty([info objectForKey:@"country_code"]) ? [info objectForKey:@"country_code"]:@""  completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}

-(void)sendEmailCodeForRegisterWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] sendRegVCode:!BL_IsStrEmpty([info objectForKey:@"email"]) ? [info objectForKey:@"email"]:@"" completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
-(void)registerAccountWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] regist:!BL_IsStrEmpty([info objectForKey:@"account"]) ? [info objectForKey:@"account"]:@""  password:!BL_IsStrEmpty([info objectForKey:@"password"]) ? [info objectForKey:@"password"]:@""  nickname:!BL_IsStrEmpty([info objectForKey:@"nickname"]) ? [info objectForKey:@"nickname"]:@""  vcode:!BL_IsStrEmpty([info objectForKey:@"code"]) ? [info objectForKey:@"code"]:@"" sex:BL_ACCOUNT_MALE birthday:!BL_IsStrEmpty([info objectForKey:@"birthday"]) ? [info objectForKey:@"birthday"]:@""  countryCode:!BL_IsStrEmpty([info objectForKey:@"country_code"]) ? [info objectForKey:@"country_code"]:@""  iconPath:!BL_IsStrEmpty([info objectForKey:@"iconpath"]) ? [info objectForKey:@"iconpath"]:@""  completionHandler:^(BLLoginResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg,
                @"data":@{//nickname,icon_path,sex,country_code,phone,email,birthday,user_type
                        @"login_session":result.loginsession?result.loginsession:@"",
                        @"user_id":result.userid?result.userid:@"",
                        @"nickname":result.nickname?result.nickname:@"",
                        @"icon_path":result.iconpath?result.iconpath:@"",
                        @"sex":result.sex?result.sex:@"",
                        @"country_code":result.countrycode?result.countrycode:@"",
                        @"phone":result.phone?result.phone:@"",
                        @"email":result.email?result.email:@"",
                        @"birthday":result.birthday?result.birthday:@"",
                        @"user_type":result.usertype?result.usertype:@"",
                }
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//重置密码发送验证码
-(void)resetPwdSendVCodeWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] sendRetriveVCode:!BL_IsStrEmpty([info objectForKey:@"account"]) ? [info objectForKey:@"account"]:@""  completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//重置密码
-(void)resetPwdWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    [[BLAccount sharedAccount] retrivePassword:!BL_IsStrEmpty([info objectForKey:@"account"]) ? [info objectForKey:@"account"]:@""  vcode:!BL_IsStrEmpty([info objectForKey:@"vcode"]) ? [info objectForKey:@"vcode"]:@"" newPassword:!BL_IsStrEmpty([info objectForKey:@"password"]) ? [info objectForKey:@"password"]:@"" completionHandler:^(BLLoginResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg,
                @"data":@{//nickname,icon_path,sex,country_code,phone,email,birthday,user_type
                        @"login_session":result.loginsession?result.loginsession:@"",
                        @"user_id":result.userid?result.userid:@"",
                        @"nickname":result.nickname?result.nickname:@"",
                        @"icon_path":result.iconpath?result.iconpath:@"",
                        @"sex":result.sex?result.sex:@"",
                        @"country_code":result.countrycode?result.countrycode:@"",
                        @"phone":result.phone?result.phone:@"",
                        @"email":result.email?result.email:@"",
                        @"birthday":result.birthday?result.birthday:@"",
                        @"user_type":result.usertype?result.usertype:@"",
                }
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//发送修改手机号的验证码，
-(void)sendVCodeToModifyPhoneWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] sendModifyVCode:!BL_IsStrEmpty([info objectForKey:@"phone"]) ? ([info objectForKey:@"phone"]):@"" countryCode:!BL_IsStrEmpty([info objectForKey:@"country_code"]) ? [info objectForKey:@"country_code"]:@"" completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//发送修改邮箱的验证码
-(void)sendVCodeToModifyEmailWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    [[BLAccount sharedAccount] sendModifyVCode:!BL_IsStrEmpty([info objectForKey:@"email"]) ? [info objectForKey:@"email"]:@"" completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//修改手机号
-(void)modifyPhoneWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    [[BLAccount sharedAccount] modifyPhone:!BL_IsStrEmpty([info objectForKey:@"phone"]) ? [info objectForKey:@"phone"]:@"" countryCode:!BL_IsStrEmpty([info objectForKey:@"country_code"]) ? [info objectForKey:@"country_code"]:@"" vcode:!BL_IsStrEmpty([info objectForKey:@"vcode"]) ? [info objectForKey:@"vcode"]:@"" password:!BL_IsStrEmpty([info objectForKey:@"password"]) ? [info objectForKey:@"password"]:@""  newpassword:nil completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//修改邮箱
-(void)modifyEmailWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    [[BLAccount sharedAccount] modifyEmail:!BL_IsStrEmpty([info objectForKey:@"email"]) ? [info objectForKey:@"email"]:@"" vcode:!BL_IsStrEmpty([info objectForKey:@"vcode"]) ?[info objectForKey:@"vcode"]:@"" password:!BL_IsStrEmpty([info objectForKey:@"password"]) ? [info objectForKey:@"password"]:@"" newpassword:nil completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//修改密码
-(void)modifyPwdWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    [[BLAccount sharedAccount] modifyPassword:!BL_IsStrEmpty([info objectForKey:@"old_password"]) ? [info objectForKey:@"old_password"]:@"" newPassword:!BL_IsStrEmpty([info objectForKey:@"new_password"]) ? [info objectForKey:@"new_password"]:@"" completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//修改昵称
-(void)modifyNickNameWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    [[BLAccount sharedAccount] modifyUserNickname:!BL_IsStrEmpty([info objectForKey:@"nicknam"]) ? [info objectForKey:@"nicknam"]:@"" completionHandler:^(BLBaseResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
//修改头像
-(void)modifyIncopathWithInfo:(NSDictionary *)info uccess:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    [[BLAccount sharedAccount] modifyUserIcon:!BL_IsStrEmpty([info objectForKey:@"file_path"]) ? [info objectForKey:@"file_path"]:@"" completionHandler:^(BLModifyUserIconResult * _Nonnull result) {
        if ([result succeed]) {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            success(dic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}
-(void)getUserInfoWithInfo:(NSDictionary *)info uccess:(successIdWithBlock)success failure:(failureIdWithBlock)failure {
    NSArray *userIdArr = [self toArrayOrNSDictionary:[info objectForKey:@"data"]];
    [[BLAccount sharedAccount] getUserInfo:userIdArr.count > 0 ? userIdArr:@[] completionHandler:^(BLGetUserInfoResult * _Nonnull result) {
        if ([result succeed]) {
            NSMutableArray *array = [NSMutableArray array];
            for (BLUserInfo *userInfo in result.info) {
                NSDictionary *dic = @{
                    @"user_id":userInfo.userid?userInfo.userid:@"",
                    @"nickname":userInfo.nickname?userInfo.nickname:@"",
                    @"icon":userInfo.iconUrl?userInfo.iconUrl:@""
                };
                [array addObject:dic];
            }
            NSDictionary *resultDic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@"",
                @"data":array
            };
            success(resultDic);
        }else {
            NSDictionary *dic = @{
                @"status":@(result.status),
                @"msg":result.msg?result.msg:@""
            };
            failure(dic);
        }
    }];
}









- (id)toArrayOrNSDictionary:(NSString *)jsonData{

    if (jsonData != nil) {
        NSData* data = [jsonData dataUsingEncoding:NSUTF8StringEncoding];
        id jsonObject = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingAllowFragments error:nil];
        if (jsonObject != nil){
            return jsonObject;
        }else{
            // 解析错误
            return nil;

        }
    }
    return nil;
}



- (NSString *)p_toJsonString:(NSDictionary *)dic {
    if (dic == nil) {
        return @"";
    }
    NSError *error;
    NSData *data = [NSJSONSerialization dataWithJSONObject:dic options:0 error:&error];
    if (error) {
        NSLog(@"%@", error.localizedDescription);
        return nil;
    }

    return [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
}

- (NSDictionary *)p_toDictory:(id)json {
    if (!json || json == (id)kCFNull) return nil;
    NSDictionary *dic = nil;
    NSData *jsonData = nil;
    if ([json isKindOfClass:[NSDictionary class]]) {
        dic = json;
    } else if ([json isKindOfClass:[NSString class]]) {
        jsonData = [(NSString *)json dataUsingEncoding : NSUTF8StringEncoding];
    } else if ([json isKindOfClass:[NSData class]]) {
        jsonData = json;
    }
    if (jsonData) {
        dic = [NSJSONSerialization JSONObjectWithData:jsonData options:kNilOptions error:NULL];
        if (![dic isKindOfClass:[NSDictionary class]]) dic = nil;
    }
    return dic;
}
#pragma mark - BLControllerDelegate
- (Boolean)filterDevice:(BLDNADevice *)device {
    return NO;
}

- (Boolean)shouldAdd:(BLDNADevice *)device {
    //本地搜索到设备不会自动添加到SDK内
    return NO;
}

- (void)onDeviceUpdate:(BLDNADevice *)device isNewDevice:(Boolean)isNewDevice {
    BOOL isContain = NO;
    for (BLDNADevice *tempDevice in self.localDeviceArray) {
        if ([tempDevice.did isEqualToString:device.did]) {
            isContain = YES;
            tempDevice.controlId = device.controlId;
            tempDevice.controlKey = device.controlKey;
            tempDevice.lanaddr = device.lanaddr;
            tempDevice.newConfig = device.newConfig;
            break;
        }
    }
    if (!isContain) {
        [self.localDeviceArray addObject:device];
    }
}

- (NSString *)sharedPath {
    BLConfigParam *config = [BLConfigParam sharedConfigParam];
    return config.sdkFileSavePath;
}


@end
